@@ -16,11 +16,11 @@ See the License for the specific language governing permissions and
 limitations under the License.
 
 :license: Apache License 2.0
-:version: 0.1.9
+:version: 0.2.1
 """
 
 # Module version
-__version_info__ = (0, 2, 0)
+__version_info__ = (0, 2, 1)
 __version__ = ".".join(str(x) for x in __version_info__)
 
 # Documentation strings format
@@ -117,13 +117,18 @@ def dump(obj, serialize_method=None, ignore_attribute=None, ignore=None,
     # Parse / return default "types"...
     # Apply additional types, override built-in types
     # (reminder: config.serialize_handlers is a dict)
-    if isinstance(obj, tuple(config.serialize_handlers)):
-        return config.serialize_handlers[type(obj)](obj, serialize_method,
-                                                    ignore_attribute, ignore,
-                                                    config)
+    try:
+        serializer = config.serialize_handlers[type(obj)]
+    except KeyError:
+        # Not a serializer
+        pass
+    else:
+        if serializer is not None:
+            return serializer(obj, serialize_method, ignore_attribute,
+                              ignore, config)
 
     # Primitive
-    elif isinstance(obj, utils.primitive_types):
+    if isinstance(obj, utils.primitive_types):
         return obj
 
     # Iterative
